<?php

function htmlEscape($str)
{
	return htmlspecialchars($str, ENT_QUOTES, 'UTF-8');
}

function chainAsData($chain)
{
	return '$data[\'' . implode('\'][\'', explode('.', $chain)) . '\']';
}

function parseTemplate($fileTpl, &$data = NULL)
{
	$timeT = @filemtime($fileTpl);
	if (!$timeT)
		return;

	$filePhp = $fileTpl . '.php';
	$timeP = @filemtime($filePhp);

	// PHP file is absent or obsolete
	if (!$timeP || $timeP < $timeT)
	{
		// get template & split it into parts divided by "{macro}" or "<!--macro-->"
		$text = file_get_contents($fileTpl);
		$parts = preg_split('/(\{.*\})|(<!--.*-->)/U', $text, -1, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY);

		if (count($parts) > 1)
		{
			$text = '';
			foreach ($parts as &$str)
			{
				if ($str{0} == '{' || strpos($str, '<!--') === 0 )
				{
					$subs = $escape = NULL;
					$macro = trim(($str{0} == '{') ? substr($str, 1, -1) : substr($str, 4, -3));

					if (strlen($macro) > 0)
						switch ($macro{0})
						{
							case '%': // -- operator
								if (stripos($macro, '%FOREACH') === 0)
								{
									if (strpos($macro, '=>') !== FALSE)
									{
										if (preg_match('/^%FOREACH\s+([^\s]+)\s+AS\s+([^\s]+)\s*=>\s*([^\s]+)$/i', $macro, $match))
											$subs = 'foreach (' . chainAsData($match[1]) . ' as ' . chainAsData($match[2]) .
													' => &' . chainAsData($match[3]) . ') {';
									}
									else
									{
										if (preg_match('/^%FOREACH\s+([^\s]+)\s+AS\s+([^\s]+)$/i', $macro, $match))
											$subs = 'foreach (' . chainAsData($match[1]) . ' as &' . chainAsData($match[2]) . ') {';
									}
								}
								else if (stripos($macro, '%IF') === 0 || stripos($macro, '%ELSEIF') === 0)
								{
									if (preg_match('/^%(ELSE)?IF\s+([^\s]+)$/i', $macro, $match))
										$subs = ((!empty($match[1])) ? '} else ' : '') . 'if (!empty('.chainAsData($match[2]).')) {';
								}
								else if (strcasecmp($macro, '%ELSE') == 0)
								{
									$subs = '} else {';
								}
								else if (strcasecmp($macro, '%END') == 0)
								{
									$subs = '}';
								}
								else if (stripos($macro, '%INCLUDE') === 0)
								{
									if (preg_match('/^%INCLUDE\s+([^\s]+)$/i', $macro, $match))
									{
										$fileSubTpl = dirname($fileTpl) . '/' . trim(substr($macro, 8)) . '.tpl';
										$subs = 'parseTemplate(\'' . $fileSubTpl . '\', $data)';
									}
								}
								break;

							case '@': // -- var substitution (w. markup filter)
								$escape = TRUE;

							case ':': // -- var substitution (simple)
								$subs = substr($macro, 1);

								// is any var modifier?
								if(strpos($subs, '|') === FALSE)
									$subs = chainAsData($subs);
								else
								{
									$mods = explode('|', $subs);
									$subs = chainAsData(array_shift($mods));
									foreach ($mods as $mod)
										if(($p = strpos($mod, ',')) !== FALSE)
											$subs = substr($mod, 0, $p) . '(' . $subs . substr($mod, $p) . ')';
										else
											$subs = $mod . '(' . $subs . ')';
								}

								if ($escape)
									$subs = 'htmlEscape(' . $subs . ')';

								$subs = 'echo ' . $subs;
								break;
						} // switch $macro{0}

					if (!is_null($subs))
						$str = '<?php ' . $subs . ' ?>';
				} // if key

				$text .= $str;
			} // foreach parts
		}

		file_put_contents($filePhp, $text);
		$timeP = 1;
	} // if PHP file absent or obsolete

	if ($timeP)
		include $filePhp;
}